#! -*- coding: utf-8 -*-

import base64
import mimetypes
from urllib import urlencode
from urlparse import urlparse
from urlparse import urlunparse
# For Python < 2.6
from django.contrib.sites.models import Site
from django.template.context import RequestContext
from jumbo.shortcuts import render_to_response
from mailer.models import  LinkStatus, Link, Attachment
from mailer.settings import USE_UTM_TAGS, TRACKING_LINKS
from mailer.utils import render_string
from mailer.utils.mail_utils import track_links
from mailer.utils.tokens import untokenize

try:
    from urlparse import parse_qs
except ImportError:
    from cgi import parse_qs

from django.http import HttpResponse, Http404
from django.http import HttpResponseRedirect
from django.utils.encoding import smart_str

from jumbo.utils.functions import get_settings

from mailer.models import  MailingStatus


#@login_required
#def view_newsletter_preview(request, slug):
#    """View of the newsletter preview"""
#    context = {'contact': request.user}
#    return render_newsletter(request, slug, context)

# ATTACH
#@login_required
def mailer_email_attachment(request, obj_id):
    """Download attachment"""
    try:
        attachment = Attachment.objects.get(id=obj_id)
        ctype, encoding = mimetypes.guess_type(attachment.file_attachment.path)
        if ctype is None or encoding is not None:
            ctype = 'application/octet-stream'
        response = HttpResponse(attachment.file_attachment.read())
        response['Content-Type'] = ctype
        response['Content-Disposition'] = 'attachment; filename=%s' % attachment.file_attachment.name
        response['Content-Length'] = str(attachment.file_attachment.size)
        if encoding is not None:
            response['Content-Encoding'] = encoding
        response.write(attachment.file_attachment)
    except:
        raise Http404
    return response

# EMAIL
def mailer_email_render_email(request, uidb36, token):
#    """Return an email in HTML format"""
    email_object = untokenize(uidb36, token)
    mail_status = MailingStatus.objects.create(
        email_object=email_object,
        status=MailingStatus.OPENED_ON_SITE
    )

    context = {
        'email': email_object,
        'uidb36': uidb36, 'token': token,
        'domain': Site.objects.get_current().domain
    }

    content = render_string(email_object.content, context)
    subject = render_string(email_object.subject, context)
    if TRACKING_LINKS:
        content = track_links(content, context)

    return render_to_response(
        'mailer/email_detail.html', content=content, subject=subject, object=email_object,
        context_instance=RequestContext(request)
    )

# TRACKING
def mailer_email_tracking(request, uidb36, token):
    """Track the opening of the newsletter by requesting a blank img"""
    TRACKING_IMAGE = get_settings("mailer.settings", "TRACKING_IMAGE")
    try:
        email_object = untokenize(uidb36, token)
        mail_status = MailingStatus.objects.create(
            email_object=email_object,
            status=MailingStatus.OPENED
        )
    except:
        raise Http404
    return HttpResponse(base64.b64decode(TRACKING_IMAGE), mimetype='image/png')

def mailer_email_tracking_link(request, uidb36, token, link_id):
#    """Track the opening of a link on the website"""
#    print >> sys.stderr, "uidb36: %s" % uidb36
    try:
        email_object = untokenize(uidb36, token)
        mailing_status = MailingStatus.objects.create(
            email_object=email_object,
            status=MailingStatus.LINK_OPENED
        )
    except:
        raise Http404

    try:
        link = Link.objects.get(id = link_id)
    except Exception, e:
        raise Http404

    link_status = LinkStatus(mailing_status=mailing_status, link=link)
    link_status.save()

    if not USE_UTM_TAGS:
        return HttpResponseRedirect(link.url)

    url_parts = urlparse(link.url)
    query_dict = parse_qs(url_parts.query)
    query_dict.update({
        'utm_source': 'email_%s' % email_object.pk,
        'utm_medium': 'mail',
        'utm_campaign': smart_str(email_object.subject)
    })
    url = urlunparse((
        url_parts.scheme, url_parts.netloc, url_parts.path,
        url_parts.params, urlencode(query_dict), url_parts.fragment
    ))

    return HttpResponseRedirect(url)
