#! -*- coding: utf-8 -*-

import csv, sys
from datetime import timedelta, datetime, date
import base64
from urllib import urlencode
from urlparse import urlparse
from urlparse import urlunparse
# For Python < 2.6
try:
    from urlparse import parse_qs
except ImportError:
    from cgi import parse_qs

from django.template import RequestContext
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.contrib.sites.models import Site
from django.contrib.auth.decorators import login_required
from django.template.loader import render_to_string as render_file
from django.db.models import Q
from django.http import HttpResponse, Http404
from django.http import HttpResponseRedirect
from django.utils.encoding import smart_str
from django.utils.translation import ugettext as _
from django.template.defaultfilters import date
from django.utils.encoding import smart_str

from newsletter.models import Newsletter
from newsletter.models import MailingList
from newsletter.utils import render_string
from newsletter.utils.nl_utils import body_insertion
from newsletter.utils.nl_utils import track_links
from newsletter.utils.tokens import untokenize
#from newsletter.utils.ofc import Chart
from newsletter.settings import TRACKING_LINKS, INCLUDE_UNSUBSCRIPTION
from newsletter.settings import USE_UTM_TAGS
from newsletter.settings import TRACKING_IMAGE
#from newsletter.utils.statistics import get_newsletter_top_links
#from newsletter.utils.statistics import get_newsletter_statistics
#from newsletter.utils.statistics import get_newsletter_opening_statistics
#from newsletter.utils.statistics import get_newsletter_clicked_link_statistics
#from newsletter.forms import CreditForm

# NEWSLETTER
def render_newsletter(request, slug, context):
    """Return a newsletter in HTML format"""
    newsletter = get_object_or_404(Newsletter, slug=slug)
    context.update({
        'newsletter': newsletter,
        'domain': Site.objects.get_current().domain
    })

    content = render_string(newsletter.content, context)
    title = render_string(newsletter.title, context)
    if TRACKING_LINKS:
        content = track_links(content, context)

    if INCLUDE_UNSUBSCRIPTION:
        unsubscription = render_file(
            'newsletter/newsletter_link_unsubscribe.html', context
        )
        content = body_insertion(content, unsubscription, end=True)

    return render_to_response(
        'newsletter/newsletter_detail.html',
            {'content': content, 'title': title, 'object': newsletter},
        context_instance=RequestContext(request)
    )

@login_required
def view_newsletter_preview(request, slug):
    """View of the newsletter preview"""
    context = {'contact': request.user}
    return render_newsletter(request, slug, context)

def view_newsletter_contact(request, slug, uidb36, token):
    """Visualization of a newsletter by an user"""
    newsletter = get_object_or_404(Newsletter, slug=slug)
    contact = untokenize(uidb36, token)
    try:
        cms = ContactMailingStatus.objects.filter(
            newsletter=newsletter,
            contact=contact,
            status=0
        ).order_by('-id')
        if cms:
            cms = cms[0]
        cms.opened_on_site = cms.opened_on_site + 1
        if not cms.first_open_date:
            cms.first_open_date = datetime.now()
        cms.save()
    except:
        raise Http404

    context = {
        'contact': contact, 'uidb36': uidb36, 'token': token
    }

    return render_newsletter(request, slug, context)

# MAILING LIST

def view_mailinglist_unsubscribe(request, slug, uidb36, token):
    """Unsubscribe a contact to a mailing list"""
    newsletter = get_object_or_404(Newsletter, slug=slug)
    contact = untokenize(uidb36, token)

    already_unsubscribed = contact in newsletter.mailing_list.unsubscribers.all()

    if request.POST.get('email') and not already_unsubscribed:
        newsletter.mailing_list.unsubscribers.add(contact)
        newsletter.mailing_list.save()
        already_unsubscribed = True

        try:
            cms = ContactMailingStatus.objects.filter(
                newsletter=newsletter,
                contact=contact,
                status=0
            ).order_by('-id')
            if cms:
                cms = cms[0]
            cms.unsubscription = True
            if not cms.unsubscription_date:
                cms.unsubscription_date = datetime.now()
            cms.save()
        except:
            raise Http404

    return render_to_response(
        'newsletter/mailing_list_unsubscribe.html',
            {'email': contact.email, 'already_unsubscribed': already_unsubscribed},
        context_instance=RequestContext(request)
    )

def view_mailinglist_subscribe(request, form_class, mailing_list_id=None):
    """
    A simple view that shows a form for subscription
    for a mailing list(s).
    """
    subscribed = False
    mailing_list = None
    if mailing_list_id:
        mailing_list = get_object_or_404(MailingList, id=mailing_list_id)

    if request.POST and not subscribed:
        form = form_class(request.POST)
        if form.is_valid():
            form.save(mailing_list)
            subscribed = True
    else:
        form = form_class()
        print form

    return render_to_response(
        'newsletter/mailing_list_subscribe.html',
            {'subscribed': subscribed, 'mailing_list': mailing_list, 'form': form},
        context_instance=RequestContext(request)
    )

